@@ -14,11 +14,13 @@
  *
  * Search engines consider www.somedomain.com and somedomain.com to be two
  * different URLs when they both go to the same location. This SEO enhancement
- * prevents penality for duplicate content by redirecting all incoming links to
+ * prevents penalty for duplicate content by redirecting all incoming links to
  * one or the other.
  *
  * Prevents redirection for feeds, trackbacks, searches, comment popup, and
- * admin URLs. Does not redirect on IIS, page/post previews, and on form data.
+ * admin URLs. Does not redirect on non-pretty-permalink-supporting IIS 7+,
+ * page/post previews, WP admin, Trackbacks, robots.txt, searches, or on POST
+ * requests.
  *
  * Will also attempt to find the correct link when a user enters a URL that does
  * not exist based on exact WordPress query. Will instead try to parse the URL
@@ -37,8 +39,23 @@
 function redirect_canonical( $requested_url = null, $do_redirect = true ) {
 	global $wp_rewrite, $is_IIS, $wp_query, $wpdb;
 
-	if ( is_trackback() || is_search() || is_comments_popup() || is_admin() || !empty($_POST) || is_preview() || is_robots() || $is_IIS )
+	if ( isset( $_SERVER['REQUEST_METHOD'] ) && ! in_array( strtoupper( $_SERVER['REQUEST_METHOD'] ), array( 'GET', 'HEAD' ) ) ) {
 		return;
+	}
+
+	// If we're not in wp-admin and the post has been published and preview nonce
+	// is non-existent or invalid then no need for preview in query
+	if ( is_preview() && get_query_var( 'p' ) && 'publish' == get_post_status( get_query_var( 'p' ) ) ) {
+		if ( ! isset( $_GET['preview_id'] )
+			|| ! isset( $_GET['preview_nonce'] )
+			|| ! wp_verify_nonce( $_GET['preview_nonce'], 'post_preview_' . (int) $_GET['preview_id'] ) ) {
+			$wp_query->is_preview = false;
+		}
+	}
+
+	if ( is_trackback() || is_search() || is_comments_popup() || is_admin() || is_preview() || is_robots() || ( $is_IIS && !iis7_supports_permalinks() ) ) {
+		return;
+	}
 
 	if ( !$requested_url ) {
 		// build the URL in the address bar
@@ -66,6 +83,18 @@ function redirect_canonical( $requested_url = null, $do_redirect = true ) {
 	if ( !isset($redirect['query']) )
 		$redirect['query'] = '';
 
+	// It's not a preview, so remove it from URL
+	if ( get_query_var( 'preview' ) ) {
+		$redirect['query'] = remove_query_arg( 'preview', $redirect['query'] );
+	}
+
+	if ( is_feed() && ( $id = get_query_var( 'p' ) ) ) {
+		if ( $redirect_url = get_post_comments_feed_link( $id, get_query_var( 'feed' ) ) ) {
+			$redirect['query'] = _remove_qs_args_if_not_in_url( $redirect['query'], array( 'p', 'page_id', 'attachment_id', 'pagename', 'name', 'post_type', 'feed'), $redirect_url );
+			$redirect['path'] = parse_url( $redirect_url, PHP_URL_PATH );
+		}
+	}
+
 	if ( is_singular() && 1 > $wp_query->post_count && ($id = get_query_var('p')) ) {
 
 		$vars = $wpdb->get_results( $wpdb->prepare("SELECT post_type, post_parent FROM $wpdb->posts WHERE ID = %d", $id) );
@@ -75,7 +104,7 @@ function redirect_canonical( $requested_url = null, $do_redirect = true ) {
 				$id = $vars->post_parent;
 
 			if ( $redirect_url = get_permalink($id) )
-				$redirect['query'] = remove_query_arg(array('p', 'page_id', 'attachment_id', 'post_type'), $redirect['query']);
+				$redirect['query'] = _remove_qs_args_if_not_in_url( $redirect['query'], array( 'p', 'page_id', 'attachment_id', 'pagename', 'name', 'post_type' ), $redirect_url );
 		}
 	}
 
@@ -88,12 +117,29 @@ function redirect_canonical( $requested_url = null, $do_redirect = true ) {
 			$post_type_obj = get_post_type_object($redirect_post->post_type);
 			if ( $post_type_obj->public ) {
 				$redirect_url = get_permalink($redirect_post);
-				$redirect['query'] = remove_query_arg(array('p', 'page_id', 'attachment_id', 'post_type'), $redirect['query']);
+				$redirect['query'] = _remove_qs_args_if_not_in_url( $redirect['query'], array( 'p', 'page_id', 'attachment_id', 'pagename', 'name', 'post_type' ), $redirect_url );
 			}
 		}
 
-		if ( ! $redirect_url )
-			$redirect_url = redirect_guess_404_permalink();
+		if ( get_query_var( 'day' ) && get_query_var( 'monthnum' ) && get_query_var( 'year' ) ) {
+			$year  = get_query_var( 'year' );
+			$month = get_query_var( 'monthnum' );
+			$day   = get_query_var( 'day' );
+			$date  = sprintf( '%04d-%02d-%02d', $year, $month, $day );
+			if ( ! wp_checkdate( $month, $day, $year, $date ) ) {
+				$redirect_url = get_month_link( $year, $month );
+				$redirect['query'] = _remove_qs_args_if_not_in_url( $redirect['query'], array( 'year', 'monthnum', 'day' ), $redirect_url );
+			}
+		} elseif ( get_query_var( 'monthnum' ) && get_query_var( 'year' ) && 12 < get_query_var( 'monthnum' ) ) {
+			$redirect_url = get_year_link( get_query_var( 'year' ) );
+			$redirect['query'] = _remove_qs_args_if_not_in_url( $redirect['query'], array( 'year', 'monthnum' ), $redirect_url );
+		}
+
+		if ( ! $redirect_url ) {
+			if ( $redirect_url = redirect_guess_404_permalink() ) {
+				$redirect['query'] = _remove_qs_args_if_not_in_url( $redirect['query'], array( 'page', 'feed', 'p', 'page_id', 'attachment_id', 'pagename', 'name', 'post_type' ), $redirect_url );
+			}
+		}
 
 	} elseif ( is_object($wp_rewrite) && $wp_rewrite->using_permalinks() ) {
 		// rewriting of old ?p=X, ?m=2004, ?m=200401, ?m=20040101
@@ -152,37 +198,56 @@ function redirect_canonical( $requested_url = null, $do_redirect = true ) {
 				$term_count += count( $tax_query['terms'] );
 
 			$obj = $wp_query->get_queried_object();
-			if ( $term_count <= 1 && !empty($obj->term_id) && ( $tax_url = get_term_link((int)$obj->term_id, $obj->taxonomy) ) && !is_wp_error($tax_url) && !empty($redirect['query']) ) {
+			if ( $term_count <= 1 && !empty($obj->term_id) && ( $tax_url = get_term_link((int)$obj->term_id, $obj->taxonomy) ) && !is_wp_error($tax_url) ) {
 				if ( !empty($redirect['query']) ) {
+					// Strip taxonomy query vars off the url.
+					$qv_remove = array( 'term', 'taxonomy');
 					if ( is_category() ) {
-						$redirect['query'] = remove_query_arg( array( 'category_name', 'category', 'cat'), $redirect['query']);
+						$qv_remove[] = 'category_name';
+						$qv_remove[] = 'cat';
 					} elseif ( is_tag() ) {
-						$redirect['query'] = remove_query_arg( array( 'tag', 'tag_id'), $redirect['query']);
-					} elseif ( is_tax() ) { // Custom taxonomies will have a custom query var, remove those too:
-						$tax = get_taxonomy( $obj->taxonomy );
-						if ( false !== $tax->query_var)
-							$redirect['query'] = remove_query_arg($tax->query_var, $redirect['query']);
-						else
-							$redirect['query'] = remove_query_arg( array( 'term', 'taxonomy'), $redirect['query']);
+						$qv_remove[] = 'tag';
+						$qv_remove[] = 'tag_id';
+					} else { // Custom taxonomies will have a custom query var, remove those too:
+						$tax_obj = get_taxonomy( $obj->taxonomy );
+						if ( false !== $tax_obj->query_var )
+							$qv_remove[] = $tax_obj->query_var;
+					}
+
+					$rewrite_vars = array_diff( array_keys($wp_query->query), array_keys($_GET) );
+
+					if ( !array_diff($rewrite_vars, array_keys($_GET))  ) { // Check to see if all the Query vars are coming from the rewrite, none are set via $_GET
+						$redirect['query'] = remove_query_arg($qv_remove, $redirect['query']); //Remove all of the per-tax qv's
+
+						// Create the destination url for this taxonomy
+						$tax_url = parse_url($tax_url);
+						if ( ! empty($tax_url['query']) ) { // Taxonomy accessible via ?taxonomy=..&term=.. or any custom qv..
+							parse_str($tax_url['query'], $query_vars);
+							$redirect['query'] = add_query_arg($query_vars, $redirect['query']);
+						} else { // Taxonomy is accessible via a "pretty-URL"
+							$redirect['path'] = $tax_url['path'];
+						}
+
+					} else { // Some query vars are set via $_GET. Unset those from $_GET that exist via the rewrite
+						foreach ( $qv_remove as $_qv ) {
+							if ( isset($rewrite_vars[$_qv]) )
+								$redirect['query'] = remove_query_arg($_qv, $redirect['query']);
+						}
 					}
 				}
-				$tax_url = parse_url($tax_url);
-				if ( ! empty($tax_url['query']) ) { // Custom taxonomies may only be accessable via ?taxonomy=..&term=..
-					parse_str($tax_url['query'], $query_vars);
-					$redirect['query'] = add_query_arg($query_vars, $redirect['query']);
-				} else { // Taxonomy is accessable via a "pretty-URL"
-					$redirect['path'] = $tax_url['path'];
-				}
+
 			}
-		} elseif ( is_single() && strpos($wp_rewrite->permalink_structure, '%category%') !== false ) {
-			$category = get_category_by_path(get_query_var('category_name'));
+		} elseif ( is_single() && strpos($wp_rewrite->permalink_structure, '%category%') !== false && $cat = get_query_var( 'category_name' ) ) {
+			$category = get_category_by_path( $cat );
 			$post_terms = wp_get_object_terms($wp_query->get_queried_object_id(), 'category', array('fields' => 'tt_ids'));
 			if ( (!$category || is_wp_error($category)) || ( !is_wp_error($post_terms) && !empty($post_terms) && !in_array($category->term_taxonomy_id, $post_terms) ) )
 				$redirect_url = get_permalink($wp_query->get_queried_object_id());
 		}
 
 		// Post Paging
-		if ( is_singular() && get_query_var('page') && $redirect_url ) {
+		if ( is_singular() && ! is_front_page() && get_query_var('page') ) {
+			if ( !$redirect_url )
+				$redirect_url = get_permalink( get_queried_object_id() );
 			$redirect_url = trailingslashit( $redirect_url ) . user_trailingslashit( get_query_var( 'page' ), 'single_paged' );
 			$redirect['query'] = remove_query_arg( 'page', $redirect['query'] );
 		}
@@ -199,10 +264,27 @@ function redirect_canonical( $requested_url = null, $do_redirect = true ) {
 			$addl_path = '';
 			if ( is_feed() && in_array( get_query_var('feed'), $wp_rewrite->feeds ) ) {
 				$addl_path = !empty( $addl_path ) ? trailingslashit($addl_path) : '';
-				if ( get_query_var( 'withcomments' ) )
+				if ( !is_singular() && get_query_var( 'withcomments' ) )
 					$addl_path .= 'comments/';
-				$addl_path .= user_trailingslashit( 'feed/' . ( ( get_default_feed() ==  get_query_var('feed') || 'feed' == get_query_var('feed') ) ? '' : get_query_var('feed') ), 'feed' );
+				if ( ( 'rss' == get_default_feed() && 'feed' == get_query_var('feed') ) || 'rss' == get_query_var('feed') )
+					$addl_path .= user_trailingslashit( 'feed/' . ( ( get_default_feed() == 'rss2' ) ? '' : 'rss2' ), 'feed' );
+				else
+					$addl_path .= user_trailingslashit( 'feed/' . ( ( get_default_feed() ==  get_query_var('feed') || 'feed' == get_query_var('feed') ) ? '' : get_query_var('feed') ), 'feed' );
 				$redirect['query'] = remove_query_arg( 'feed', $redirect['query'] );
+			} elseif ( is_feed() && 'old' == get_query_var('feed') ) {
+				$old_feed_files = array(
+					'wp-atom.php'         => 'atom',
+					'wp-commentsrss2.php' => 'comments_rss2',
+					'wp-feed.php'         => get_default_feed(),
+					'wp-rdf.php'          => 'rdf',
+					'wp-rss.php'          => 'rss2',
+					'wp-rss2.php'         => 'rss2',
+				);
+				if ( isset( $old_feed_files[ basename( $redirect['path'] ) ] ) ) {
+					$redirect_url = get_feed_link( $old_feed_files[ basename( $redirect['path'] ) ] );
+					wp_redirect( $redirect_url, 301 );
+					die();
+				}
 			}
 
 			if ( get_query_var('paged') > 0 ) {
@@ -224,13 +306,25 @@ function redirect_canonical( $requested_url = null, $do_redirect = true ) {
 				$redirect['query'] = remove_query_arg( 'cpage', $redirect['query'] );
 			}
 
-			$redirect['path'] = user_trailingslashit( preg_replace('|/index.php/?$|', '/', $redirect['path']) ); // strip off trailing /index.php/
-			if ( !empty( $addl_path ) && $wp_rewrite->using_index_permalinks() && strpos($redirect['path'], '/index.php/') === false )
-				$redirect['path'] = trailingslashit($redirect['path']) . 'index.php/';
+			$redirect['path'] = user_trailingslashit( preg_replace('|/' . preg_quote( $wp_rewrite->index, '|' ) . '/?$|', '/', $redirect['path']) ); // strip off trailing /index.php/
+			if ( !empty( $addl_path ) && $wp_rewrite->using_index_permalinks() && strpos($redirect['path'], '/' . $wp_rewrite->index . '/') === false )
+				$redirect['path'] = trailingslashit($redirect['path']) . $wp_rewrite->index . '/';
 			if ( !empty( $addl_path ) )
 				$redirect['path'] = trailingslashit($redirect['path']) . $addl_path;
 			$redirect_url = $redirect['scheme'] . '://' . $redirect['host'] . $redirect['path'];
 		}
+
+		if ( 'wp-register.php' == basename( $redirect['path'] ) ) {
+			if ( is_multisite() ) {
+				/** This filter is documented in wp-login.php */
+				$redirect_url = apply_filters( 'wp_signup_location', network_site_url( 'wp-signup.php' ) );
+			} else {
+				$redirect_url = site_url( 'wp-login.php?action=register' );
+			}
+
+			wp_redirect( $redirect_url, 301 );
+			die();
+		}
 	}
 
 	// tack on any additional query vars
@@ -246,6 +340,7 @@ function redirect_canonical( $requested_url = null, $do_redirect = true ) {
 				unset( $_parsed_query['name'] );
 		}
 
+		$_parsed_query = rawurlencode_deep( $_parsed_query );
 		$redirect_url = add_query_arg( $_parsed_query, $redirect_url );
 	}
 
@@ -266,7 +361,7 @@ function redirect_canonical( $requested_url = null, $do_redirect = true ) {
 		unset($redirect['port']);
 
 	// trailing /index.php
-	$redirect['path'] = preg_replace('|/index.php/*?$|', '/', $redirect['path']);
+	$redirect['path'] = preg_replace('|/' . preg_quote( $wp_rewrite->index, '|' ) . '/*?$|', '/', $redirect['path']);
 
 	// Remove trailing spaces from the path
 	$redirect['path'] = preg_replace( '#(%20| )+$#', '', $redirect['path'] );
@@ -278,13 +373,16 @@ function redirect_canonical( $requested_url = null, $do_redirect = true ) {
 		// Clean up empty query strings
 		$redirect['query'] = trim(preg_replace( '#(^|&)(p|page_id|cat|tag)=?(&|$)#', '&', $redirect['query']), '&');
 
+		// Redirect obsolete feeds
+		$redirect['query'] = preg_replace( '#(^|&)feed=rss(&|$)#', '$1feed=rss2$2', $redirect['query'] );
+
 		// Remove redundant leading ampersands
 		$redirect['query'] = preg_replace( '#^\??&*?#', '', $redirect['query'] );
 	}
 
 	// strip /index.php/ when we're not using PATHINFO permalinks
 	if ( !$wp_rewrite->using_index_permalinks() )
-		$redirect['path'] = str_replace('/index.php/', '/', $redirect['path']);
+		$redirect['path'] = str_replace( '/' . $wp_rewrite->index . '/', '/', $redirect['path'] );
 
 	// trailing slashes
 	if ( is_object($wp_rewrite) && $wp_rewrite->using_permalinks() && !is_404() && (!is_front_page() || ( is_front_page() && (get_query_var('paged') > 1) ) ) ) {
@@ -319,7 +417,7 @@ function redirect_canonical( $requested_url = null, $do_redirect = true ) {
 		( strtolower($original['host']) != 'www.' . strtolower($redirect['host']) && 'www.' . strtolower($original['host']) != strtolower($redirect['host']) ) )
 		$redirect['host'] = $original['host'];
 
-	$compare_original = array($original['host'], $original['path']);
+	$compare_original = array( $original['host'], $original['path'] );
 
 	if ( !empty( $original['port'] ) )
 		$compare_original[] = $original['port'];
@@ -327,7 +425,7 @@ function redirect_canonical( $requested_url = null, $do_redirect = true ) {
 	if ( !empty( $original['query'] ) )
 		$compare_original[] = $original['query'];
 
-	$compare_redirect = array($redirect['host'], $redirect['path']);
+	$compare_redirect = array( $redirect['host'], $redirect['path'] );
 
 	if ( !empty( $redirect['port'] ) )
 		$compare_redirect[] = $redirect['port'];
@@ -357,8 +455,17 @@ function redirect_canonical( $requested_url = null, $do_redirect = true ) {
 		$requested_url = preg_replace_callback('|%[a-fA-F0-9][a-fA-F0-9]|', 'lowercase_octets', $requested_url);
 	}
 
-	// Note that you can use the "redirect_canonical" filter to cancel a canonical redirect for whatever reason by returning FALSE
-	$redirect_url = apply_filters('redirect_canonical', $redirect_url, $requested_url);
+	/**
+	 * Filter the canonical redirect URL.
+	 *
+	 * Returning false to this filter will cancel the redirect.
+	 *
+	 * @since 2.3.0
+	 *
+	 * @param string $redirect_url  The redirect URL.
+	 * @param string $requested_url The requested URL.
+	 */
+	$redirect_url = apply_filters( 'redirect_canonical', $redirect_url, $requested_url );
 
 	if ( !$redirect_url || $redirect_url == $requested_url ) // yes, again -- in case the filter aborted the request
 		return false;
@@ -379,38 +486,97 @@ function redirect_canonical( $requested_url = null, $do_redirect = true ) {
 }
 
 /**
- * Attempts to guess correct post based on query vars.
+ * Removes arguments from a query string if they are not present in a URL
+ * DO NOT use this in plugin code.
+ *
+ * @since 3.4.0
+ * @access private
+ *
+ * @return string The altered query string
+ */
+function _remove_qs_args_if_not_in_url( $query_string, Array $args_to_check, $url ) {
+	$parsed_url = @parse_url( $url );
+	if ( ! empty( $parsed_url['query'] ) ) {
+		parse_str( $parsed_url['query'], $parsed_query );
+		foreach ( $args_to_check as $qv ) {
+			if ( !isset( $parsed_query[$qv] ) )
+				$query_string = remove_query_arg( $qv, $query_string );
+		}
+	} else {
+		$query_string = remove_query_arg( $args_to_check, $query_string );
+	}
+	return $query_string;
+}
+
+/**
+ * Attempts to guess the correct URL based on query vars
  *
  * @since 2.3.0
  * @uses $wpdb
  *
- * @return bool|string Returns False, if it can't find post, returns correct
- *		location on success.
+ * @return bool|string The correct URL if one is found. False on failure.
  */
 function redirect_guess_404_permalink() {
-	global $wpdb;
+	global $wpdb, $wp_rewrite;
 
-	if ( !get_query_var('name') )
-		return false;
+	if ( get_query_var('name') ) {
+		$where = $wpdb->prepare("post_name LIKE %s", $wpdb->esc_like( get_query_var('name') ) . '%');
 
-	$where = $wpdb->prepare("post_name LIKE %s", like_escape( get_query_var('name') ) . '%');
+		// if any of post_type, year, monthnum, or day are set, use them to refine the query
+		if ( get_query_var('post_type') )
+			$where .= $wpdb->prepare(" AND post_type = %s", get_query_var('post_type'));
+		else
+			$where .= " AND post_type IN ('" . implode( "', '", get_post_types( array( 'public' => true ) ) ) . "')";
 
-	// if any of post_type, year, monthnum, or day are set, use them to refine the query
-	if ( get_query_var('post_type') )
-		$where .= $wpdb->prepare(" AND post_type = %s", get_query_var('post_type'));
-	if ( get_query_var('year') )
-		$where .= $wpdb->prepare(" AND YEAR(post_date) = %d", get_query_var('year'));
-	if ( get_query_var('monthnum') )
-		$where .= $wpdb->prepare(" AND MONTH(post_date) = %d", get_query_var('monthnum'));
-	if ( get_query_var('day') )
-		$where .= $wpdb->prepare(" AND DAYOFMONTH(post_date) = %d", get_query_var('day'));
+		if ( get_query_var('year') )
+			$where .= $wpdb->prepare(" AND YEAR(post_date) = %d", get_query_var('year'));
+		if ( get_query_var('monthnum') )
+			$where .= $wpdb->prepare(" AND MONTH(post_date) = %d", get_query_var('monthnum'));
+		if ( get_query_var('day') )
+			$where .= $wpdb->prepare(" AND DAYOFMONTH(post_date) = %d", get_query_var('day'));
 
-	$post_id = $wpdb->get_var("SELECT ID FROM $wpdb->posts WHERE $where AND post_status = 'publish'");
-	if ( !$post_id )
-		return false;
-	return get_permalink($post_id);
+		$post_id = $wpdb->get_var("SELECT ID FROM $wpdb->posts WHERE $where AND post_status = 'publish'");
+		if ( ! $post_id )
+			return false;
+		if ( get_query_var( 'feed' ) )
+			return get_post_comments_feed_link( $post_id, get_query_var( 'feed' ) );
+		elseif ( get_query_var( 'page' ) )
+			return trailingslashit( get_permalink( $post_id ) ) . user_trailingslashit( get_query_var( 'page' ), 'single_paged' );
+		else
+			return get_permalink( $post_id );
+	}
+
+	return false;
 }
 
 add_action('template_redirect', 'redirect_canonical');
 
-?>
+function wp_redirect_admin_locations() {
+	global $wp_rewrite;
+	if ( ! ( is_404() && $wp_rewrite->using_permalinks() ) )
+		return;
+
+	$admins = array(
+		home_url( 'wp-admin', 'relative' ),
+		home_url( 'dashboard', 'relative' ),
+		home_url( 'admin', 'relative' ),
+		site_url( 'dashboard', 'relative' ),
+		site_url( 'admin', 'relative' ),
+	);
+	if ( in_array( untrailingslashit( $_SERVER['REQUEST_URI'] ), $admins ) ) {
+		wp_redirect( admin_url() );
+		exit;
+	}
+
+	$logins = array(
+		home_url( 'wp-login.php', 'relative' ),
+		home_url( 'login', 'relative' ),
+		site_url( 'login', 'relative' ),
+	);
+	if ( in_array( untrailingslashit( $_SERVER['REQUEST_URI'] ), $logins ) ) {
+		wp_redirect( site_url( 'wp-login.php', 'login' ) );
+		exit;
+	}
+}
+
+add_action( 'template_redirect', 'wp_redirect_admin_locations', 1000 );
